/* -*- mode: c++; c-basic-offset: 3; -*- */
#include "plot_pars.hh"
#include "GDSPlot.hh"
#include <iostream>

//====================================== define the tristate string conversion
#ifdef STR_TRISTATE
inline std::ostream&
operator<<(std::ostream& out, const tri_state& ts) {
   return out << ts.state();
}
#else
inline std::ostream&
operator<<(std::ostream& out, const tri_state& ts) {
   if (!ts.is_set()) out << "unset";
   else if (ts)      out << "true";
   else              out << "false";
   return out;
}
#endif


using namespace std;

//======================================  Constructor
plot_pars::plot_pars(void)
   : mStyleId(0), mXsize(0), mYsize(0), mXmin(0), mXmax(0), mYmin(0), mYmax(0),
     mZmin(0), mZmax(0), mNBin(100), mNextColor(0), mNSmooth(0), mTimeScale(0)
{
}

//======================================  Set logbin flag
void 
plot_pars::logbin(bool lb) {
   mLogBin =lb;
}

//======================================  Set color of the next plotted trace
void 
plot_pars::set_color(int tc) {
   mNextColor = tc;
}

//======================================  Set color of the next plotted trace
void 
plot_pars::set_color_index(int cinx) {
   set_color(cinx);
}

//======================================  Set the number of histogram bins.
void
plot_pars::set_nbins(int nb) {
   mNBin = nb;
}

//======================================  Set the number of histogram bins.
void
plot_pars::set_palette(const std::string& palette) {
   mPalet = palette;
}

//======================================  Set all the parameters
void
plot_pars::set_parameters(GDSPlot& plot) const {
   if (mStyleId)          plot.set_style(mStyleId);
   plot.title(mTitle);
   if (mLogBin.is_set())  plot.logbin(mLogBin);
   if (mLogX.is_set())    plot.xlog(mLogX);
   if (mLogY.is_set())    plot.ylog(mLogY);
   if (mLogZ.is_set())    plot.zlog(mLogZ);
   plot.set_range(mXmin, mXmax, mYmin, mYmax);
   if (!mPalet.empty())   plot.set_palette(mPalet);
   if (mNSmooth)          plot.set_smooth(mNSmooth, mSmoothOpt);
   if (mStatBox.is_set()) plot.set_stat_box(mStatBox);
   if (mTimeScale)        plot.xTimeScale(mTimeScale, mXlabel);
   else                   plot.xlabel(mXlabel);
   plot.ylabel(mYlabel);
   plot.zlabel(mZlabel);
}

//======================================  Set the Axis ranges
void 
plot_pars::set_range(double xmin, double xmax, double ymin, double ymax) {
   mXmin = xmin;
   mXmax = xmax;
   mYmin = ymin;
   mYmax = ymax;
}

//======================================  Set the print size
void 
plot_pars::set_size(double xsize, double ysize) {
   mXsize = xsize;
   mYsize = ysize;
}

//======================================  Set the smoothing options
void 
plot_pars::set_smooth(int nSmooth, const std::string& smoothOpt) {
   mNSmooth   = nSmooth;
   mSmoothOpt = smoothOpt;
}

void
plot_pars::set_stat_box(bool tf) {
   mStatBox = tf;
}

//======================================  Set the plot style
void 
plot_pars::set_style(int id) {
   mStyleId = id;
}

//======================================  Print current parameter status
void
plot_pars::status(ostream&out) const {
   out << "Plot parameter block status" << endl;
   out << "style-id:      " << mStyleId << endl;
   out << "x-size:        " << mXsize << endl;
   out << "ty-size:       " << mYsize << endl;
   out << "x-minimum:     " << mXmin << endl;
   out << "x-maximum:     " << mXmax << endl;
   out << "y-minimum:     " << mYmin << endl;
   out << "y-maximum:     " << mYmax << endl;
   out << "z-minimum:     " << mZmin << endl;
   out << "z-maximum:     " << mZmax << endl;
   out << "log x-axis:    " << mLogX << endl;
   out << "log y-axis:    " << mLogY << endl;
   out << "log z-axis:    " << mLogZ << endl;
   out << "# histo bins:  " << mNBin << endl;
   out << "log bins:      " << mLogBin << endl;
   out << "stat box:      " << mStatBox << endl;
   out << "next-color:    " << mNextColor << endl;
   out << "# smoothing:   " << mNSmooth << endl;
   out << "palette name:  " << mPalet << endl;
   out << "title:         " << mTitle << endl;
   out << "x-axis label:  " << mXlabel << endl;
   out << "y-axis label:  " << mYlabel << endl;
   out << "z-axis label:  " << mZlabel << endl;
   out << "smooth option: " << mSmoothOpt << endl;
   out << "time scale:    " << mTimeScale << endl;
}

//======================================  Set canvas title
void 
plot_pars::title(const std::string& titl) {
   mTitle = titl;
}

//======================================  Set the X-axis label
void 
plot_pars::xlabel(const std::string& titl) {
   mXlabel = titl;
}

//======================================  Set X-axis log scale
void 
plot_pars::xlog(bool logx) {
   mLogX = logx;
}

//======================================  Set x time scale in reasonable units
double 
plot_pars::xTimeScale(double dT, const std::string& xttl) {
   mTimeScale = dT;
   mXlabel    = xttl;
}


//======================================  Set the Y-axis label
void 
plot_pars::ylabel(const std::string& titl) {
   mYlabel = titl;
}

//======================================  Set Y-axis log scale
void 
plot_pars::ylog(bool logy) {
   mLogY = logy;
}

//======================================  Set the Z-axis label
void 
plot_pars::zlabel(const std::string& titl) {
   mZlabel = titl;
}

//======================================  Set Y-axis log scale
void 
plot_pars::zlog(bool logz) {
   mLogZ = logz;
}
